import * as fs from 'fs'
import * as path from 'path'
import axios from 'axios'
import unzipper from 'unzipper'

import { InstallProgressPeriod, InstallTask, updateInstallTask } from './tasks'
import { randomUUID } from 'crypto'

const byteSizeUnits = ['Bytes', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB']

function formatBytesWithUnit(bytes: number, unitIndex: number, decimals = 2): string {
  if (!+bytes) return '0'

  const k = 1024
  const dm = decimals < 0 ? 0 : decimals

  return `${parseFloat((bytes / Math.pow(k, unitIndex)).toFixed(dm))}`
}

function downloadFile(task: InstallTask, url: string, output: string): Promise<void> {
  // eslint-disable-next-line no-async-promise-executor
  return new Promise(async (resolve, reject) => {
    const writer = fs.createWriteStream(output)

    const res = await axios.get(url as string, {
      responseType: 'stream',
    })

    const len = parseInt(res.headers['content-length'], 10)
    let downloaded = 0

    let lastUpdate = Date.now()

    res.data.on('data', (chunk: any) => {
      downloaded += chunk.length
      const percent = downloaded / len

      // Only update if 100ms has passed since the last update
      if (Date.now() - lastUpdate >= 100) {
        lastUpdate = Date.now()
        if (task.progress) {
          const k = 1024
          const lenUnitIndex = Math.floor(Math.log(len) / Math.log(k))

          const downloadedFormatted = formatBytesWithUnit(downloaded, lenUnitIndex)
          const lenFormatted = formatBytesWithUnit(len, lenUnitIndex)

          task.progress.value = percent
          task.progress.period = InstallProgressPeriod.Downloading
          task.progress.details = `${downloadedFormatted}/${lenFormatted} ${byteSizeUnits[lenUnitIndex]}`
          updateInstallTask(task)
        }
      }
    })

    res.data.pipe(writer)

    writer.on('finish', resolve)
    writer.on('error', reject)
  })
}

function convertAssetUri(uri: string): string {
  if (uri.startsWith('attachments://')) {
    return `https://api.sn.solsynth.dev/cgi/uc/attachments/${uri.replace('attachments://', '')}`
  }
  if (uri.startsWith('http')) {
    return uri
  }
  throw new Error('Unable handle assets uri')
}

export async function downloadAssets(task: InstallTask): Promise<string | undefined> {
  const platform = process.platform
  const asset = task.release.assets[platform]
  if (!asset) return

  const output = path.join(task.basePath, randomUUID())

  await downloadFile(task, convertAssetUri(asset.uri), output)

  const requiresExtractTypes = ['application/zip']

  if (requiresExtractTypes.includes(asset.contentType)) {
    if (task.progress) {
      task.progress.period = InstallProgressPeriod.Extracting
      task.progress.details = null
      task.progress.value = null
      updateInstallTask(task)
    }

    const dstPath = output + '-extract'

    switch (asset.contentType) {
      case 'application/zip':
        const directory = await unzipper.Open.file(output)
        await directory.extract({ path: dstPath })
        break
      default:
        throw new Error('Failed to decompress, unsupported type')
    }

    fs.unlinkSync(output)

    function moveFiles(from: string, to: string) {
      const files = fs.readdirSync(from)

      for (const file of files) {
        const sourcePath = path.join(from, file)
        const targetPath = path.join(to, file)

        if (fs.statSync(sourcePath).isDirectory()) {
          moveFiles(sourcePath, targetPath)
        } else {
          fs.renameSync(sourcePath, targetPath)
        }
      }
    }

    moveFiles(dstPath, task.basePath)

    fs.rmSync(dstPath, { recursive: true, force: true })
  }

  return output
}
