import { randomUUID } from 'crypto'
import { exec } from 'child_process'
import { join } from 'path'
import { ipcMain, WebContents } from 'electron'
import { glob } from 'glob'
import { mkdirSync, statSync, unlinkSync, readdirSync } from 'fs'

import { installTasks, InstallTask, updateInstallTask, InstallProgressPeriod } from './tasks'
import { downloadAssets } from './downloader'
import { setAppRecord } from './library'

export function initInstaller(): void {
  ipcMain.handle('install-product-release', (evt, task: string) => submitInstallTask(JSON.parse(task), evt.sender))
}

function submitInstallTask(task: InstallTask, contents: WebContents): void {
  task.id = task.id ?? randomUUID()
  task.progress = { value: 0, period: InstallProgressPeriod.Initializing, done: false, error: null }
  task.emitter = contents

  if (!statSync(task.basePath).isDirectory()) {
    return
  }
  if (!task.isOverwrite) {
    if (readdirSync(task.basePath).length > 0) {
      const newPath = join(task.basePath, task.product.name)
      mkdirSync(newPath)
      task.basePath = newPath
    }
  }

  installTasks[task.id] = task
  updateInstallTask(task)

  downloadAssets(task)
    .then(async () => {
      try {
        await runInstaller(task)

        if (task.progress) {
          task.progress.period = InstallProgressPeriod.Indexing
          task.progress.details = null
          task.progress.value = null
          updateInstallTask(task)
        }

        setAppRecord({
          id: task.id ?? randomUUID(),
          product: task.product,
          release: task.release,
          basePath: task.basePath,
        })
      } catch (err: any) {
        if (task.progress) {
          task.progress.error = err.toString()
        }
      } finally {
        if (task.progress) {
          task.progress.period = InstallProgressPeriod.Completed
          task.progress.details = null
          task.progress.value = 1
          task.progress.done = true
          updateInstallTask(task)
        }
      }
    })
    .catch((err: any) => {
      if (task.progress) {
        task.progress.error = err.toString()
        task.progress.done = true
        updateInstallTask(task)
      }
    })
}

async function runInstaller(task: InstallTask): Promise<void> {
  const platform = process.platform
  const installer = task.release.installers[platform]
  if (!installer) return

  if (task.progress) {
    task.progress.period = InstallProgressPeriod.Installing
    task.progress.details = null
    task.progress.value = null
    updateInstallTask(task)
  }

  if (installer.script) {
    if (task.progress) {
      task.progress.details = 'Running installer scripts...'
      updateInstallTask(task)
    }
    exec(installer.script, {
      cwd: installer.workdir ? join(task.basePath, installer.workdir) : task.basePath,
    })
  }

  if (installer.patches) {
    if (task.progress) {
      task.progress.details = 'Applying patches...'
      updateInstallTask(task)
    }
    for (const patch of installer.patches) {
      switch (patch.action) {
        case 'rm':
          for (const file of await glob(join(task.basePath, patch.glob), {
            signal: AbortSignal.timeout(1000 * 3),
          })) {
            unlinkSync(file)
          }
      }
    }
  }
}
