import { LocalAppRecord } from '@main/library'
import {
  Avatar,
  Box,
  Button,
  Card,
  CardContent,
  Collapse,
  Container,
  Divider,
  Grid2 as Grid,
  IconButton,
  Link,
  Stack,
  Typography,
} from '@mui/material'
import { useEffect, useMemo, useState } from 'react'
import { useNavigate, useParams } from 'react-router'
import { getAttachmentUrl, MaProduct, MaRelease, sni } from 'solar-js-sdk'
import { parseContent } from '@renderer/services/parser'

import UpdateIcon from '@mui/icons-material/Download'
import PlayIcon from '@mui/icons-material/PlayArrow'
import UninstallIcon from '@mui/icons-material/Delete'
import RefreshIcon from '@mui/icons-material/Refresh'
import ArrowBackwardIcon from '@mui/icons-material/ArrowBack'
import { InstallTask } from '@main/tasks'

export default function LibraryDetails(): JSX.Element {
  const { id } = useParams()
  const navigate = useNavigate()

  const [app, setApp] = useState<LocalAppRecord>()
  const [appProductContent, setAppProductContent] = useState<string>()
  const [appReleaseContent, setAppReleaseContent] = useState<string>()

  const [busy, setBusy] = useState(false)

  const product = useMemo(() => app?.product, [app])

  function fetchApp() {
    window.electron.ipcRenderer.invoke('get-app-library-one', id).then((res) => {
      const app = JSON.parse(res)
      if (app == null) {
        navigate('/library')
        return
      }
      return setApp(app)
    })
  }

  useEffect(() => {
    fetchApp()
  }, [])

  useEffect(() => {
    if (app) {
      parseContent(app.product.meta.introduction).then((out) => setAppProductContent(out))
      parseContent(app.release.meta.content).then((out) => setAppReleaseContent(out))
    }
  }, [app])

  const [remoteReleases, setRemoteReleases] = useState<MaRelease[]>([])

  const hasUpdate = useMemo(
    () => app && remoteReleases.length > 0 && remoteReleases[0].version != app?.release?.version,
    [app, remoteReleases],
  )

  async function fetchRemoteReleases() {
    const { data: resp } = await sni.get<{ data: MaRelease[] }>('/cgi/ma/products/' + app?.product.id + '/releases', {
      params: {
        take: 1,
      },
    })

    setRemoteReleases(resp.data)
  }

  useEffect(() => {
    if (!app) return
    fetchRemoteReleases()
  }, [app])

  async function updateApp() {
    const task: InstallTask = {
      id: app!.id,
      release: remoteReleases[0],
      product: app!.product,
      basePath: app!.basePath,
      isOverwrite: true,
    }
    window.electron.ipcRenderer.invoke('install-product-release', JSON.stringify(task))
    navigate('/tasks')
  }

  async function syncApp() {
    setBusy(true)
    try {
      const { data: productData } = await sni.get<MaProduct>('/cgi/ma/products/' + app?.product.id)
      const { data: releaseData } = await sni.get<MaRelease>(
        '/cgi/ma/products/' + app?.product.id + '/releases/' + app?.release.id,
      )

      await window.electron.ipcRenderer.invoke(
        'sync-record-app',
        app?.id,
        JSON.stringify(productData),
        JSON.stringify(releaseData),
      )

      fetchApp()
    } finally {
      setBusy(false)
    }
  }

  async function uninstallApp() {
    const yes = confirm(`Are you sure to uninstall this app ${app?.product.name} version ${app?.release.version}?`)
    if (!yes) return

    setBusy(true)
    await window.electron.ipcRenderer.invoke('uninstall-app', app?.id)
    setBusy(false)

    navigate('/library')
  }

  async function launchApp() {
    setBusy(true)
    await window.electron.ipcRenderer.invoke('launch-app', app?.id)
    setBusy(false)
  }

  async function openInstalledPath() {
    await window.electron.ipcRenderer.invoke('open-path-app', app?.id)
  }

  return (
    <>
      {product?.previews && (
        <img
          src={product.previews[0]}
          style={{ aspectRatio: 16 / 5, width: '100%', objectFit: 'cover' }}
          className="border-b border-1 pb"
        />
      )}

      <Container sx={{ py: 4 }}>
        <Grid container spacing={2}>
          <Grid size={8} sx={{ display: 'flex', flexDirection: 'column', gap: 2 }}>
            <Box>
              <Button
                size="small"
                startIcon={<ArrowBackwardIcon />}
                onClick={() => navigate('/library')}
                sx={{ mb: 1 }}
              >
                Back
              </Button>
            </Box>

            <Box>
              {product?.icon && (
                <Avatar
                  variant="rounded"
                  src={getAttachmentUrl(product.icon)}
                  sx={{ width: 64, height: 64, border: 1, borderColor: 'divider', borderRadius: 4, mb: 1, mx: '-4px' }}
                />
              )}
              <Typography variant="h5" component="h1" maxWidth="sm">
                {product?.name}
              </Typography>
              <Typography variant="body1" component="p" maxWidth="sm">
                {product?.description}
              </Typography>
            </Box>

            <Divider sx={{ mb: 2 }} />

            <Typography variant="h6" component="h2" gutterBottom>
              Release Information
            </Typography>

            {appReleaseContent && (
              <Box
                component="article"
                className="prose prose-lg dark:prose-invert"
                dangerouslySetInnerHTML={{ __html: appReleaseContent ?? '' }}
              />
            )}

            <Divider sx={{ mb: 2 }} />

            <Typography variant="h6" component="h2" gutterBottom>
              Product Introduction
            </Typography>

            {appProductContent && (
              <Box
                component="article"
                className="prose prose-lg dark:prose-invert"
                dangerouslySetInnerHTML={{ __html: appProductContent ?? '' }}
              />
            )}
          </Grid>

          <Grid size={4}>
            <Stack spacing={2} alignItems="end" sx={{ mx: -1.5 }}>
              <Box>
                <IconButton disabled={busy} color="warning" onClick={syncApp}>
                  <RefreshIcon />
                </IconButton>
                <IconButton disabled={busy} color="error" onClick={uninstallApp}>
                  <UninstallIcon />
                </IconButton>
                <IconButton disabled={busy} color="success" onClick={launchApp}>
                  <PlayIcon />
                </IconButton>
              </Box>
            </Stack>

            <Collapse in={hasUpdate}>
              <Card sx={{ mt: 2 }} variant="outlined">
                <CardContent>
                  <Typography variant="h6" component="h2" lineHeight={1.2}>
                    Update Available
                  </Typography>
                  <Typography variant="subtitle1" gutterBottom>
                    {remoteReleases[0]?.meta.title} {remoteReleases[0]?.version}
                  </Typography>

                  <Typography variant="body2">{remoteReleases[0]?.meta.description}</Typography>

                  <Button
                    variant="contained"
                    disableElevation
                    fullWidth
                    sx={{ mt: 2 }}
                    startIcon={<UpdateIcon />}
                    onClick={updateApp}
                  >
                    Update
                  </Button>
                </CardContent>
              </Card>
            </Collapse>

            <Box sx={{ mt: 4, textAlign: 'right' }}>
              <Link
                variant="caption"
                color="textSecondary"
                href={`/products/${app?.product.id}`}
                underline="none"
                display="block"
              >
                Store page
              </Link>
              <Link
                variant="caption"
                color="textSecondary"
                underline="none"
                display="block"
                sx={{ cursor: 'pointer' }}
                onClick={() => openInstalledPath()}
              >
                Open installed path
              </Link>
            </Box>

            <Box sx={{ mt: 2, textAlign: 'right' }}>
              <Typography variant="caption" component="p">
                Version {app?.release.version}
              </Typography>
              <Typography variant="caption" component="p" fontFamily="monospace" fontSize={9}>
                #{app?.id}
              </Typography>
            </Box>
          </Grid>
        </Grid>
      </Container>
    </>
  )
}
