import {
  Avatar,
  Box,
  Button,
  Card,
  CardContent,
  Container,
  FormControl,
  Grid2 as Grid,
  InputLabel,
  MenuItem,
  Select,
  Typography,
} from '@mui/material'
import { useEffect, useState } from 'react'
import { useNavigate, useParams } from 'react-router'
import { MaInstallDialog } from '@renderer/components/MaInstallDialog'
import { getAttachmentUrl, MaProduct, MaRelease, sni } from 'solar-js-sdk'

import DownloadIcon from '@mui/icons-material/Download'
import ArrowBackwardIcon from '@mui/icons-material/ArrowBack'
import { parseContent } from '@renderer/services/parser'

export default function ProductDetails(): JSX.Element {
  const { id } = useParams()
  const navigate = useNavigate()

  const [product, setProduct] = useState<MaProduct>()
  const [content, setContent] = useState<string>()

  async function fetchProduct(): Promise<void> {
    const { data } = await sni.get<MaProduct>('/cgi/ma/products/' + id)
    setProduct(data)
  }

  const [releases, setReleases] = useState<MaRelease[]>()
  const [selectedRelease, setSelectedRelease] = useState<MaRelease>()

  const [showInstaller, setShowInstaller] = useState(false)

  async function fetchReleases(): Promise<void> {
    const { data: resp } = await sni.get<{ data: MaRelease[] }>('/cgi/ma/products/' + id + '/releases', {
      params: {
        take: 10,
      },
    })

    setReleases(resp.data)
  }

  useEffect(() => {
    fetchProduct().then(() => Promise.all([fetchReleases()]))
  }, [])

  useEffect(() => {
    if (product?.meta?.introduction == null) return
    parseContent(product.meta.introduction).then((out) => setContent(out))
  }, [product?.meta?.introduction])

  return (
    <>
      {product?.previews && (
        <img
          src={product.previews[0]}
          style={{ aspectRatio: 16 / 5, width: '100%', objectFit: 'cover' }}
          className="border-b border-1 pb"
        />
      )}

      <Container sx={{ py: 4 }}>
        <Grid container spacing={2}>
          <Grid size={8} sx={{ display: 'flex', flexDirection: 'column', gap: 2 }}>
            <Box>
              <Button size="small" startIcon={<ArrowBackwardIcon />} onClick={() => navigate('/')} sx={{ mb: 1 }}>
                Back
              </Button>
            </Box>

            <Box>
              {product?.icon && (
                <Avatar
                  variant="rounded"
                  src={getAttachmentUrl(product.icon)}
                  sx={{ width: 64, height: 64, border: 1, borderColor: 'divider', borderRadius: 4, mb: 1, mx: '-4px' }}
                />
              )}
              <Typography variant="h5" component="h1" maxWidth="sm">
                {product?.name}
              </Typography>
              <Typography variant="body1" component="p" maxWidth="sm">
                {product?.description}
              </Typography>
            </Box>

            {content && (
              <Box
                component="article"
                className="prose prose-lg dark:prose-invert"
                dangerouslySetInnerHTML={{ __html: content ?? '' }}
              />
            )}
          </Grid>

          <Grid size={4}>
            <Card variant="outlined">
              <CardContent>
                <Typography variant="h6" component="h2" gutterBottom>
                  Install
                </Typography>

                <FormControl fullWidth variant="filled">
                  <InputLabel id="releases-select">Releases</InputLabel>
                  <Select
                    labelId="releases-select"
                    label="Releases"
                    value={selectedRelease?.id}
                    onChange={(evt) => setSelectedRelease(releases?.find((r) => r.id === evt.target.value))}
                  >
                    {releases?.map((r) => (
                      <MenuItem key={r.id} value={r.id}>
                        {r.meta.title}
                        <span className="opacity-50 font-mono text-xs ms-1.5 mt-0.5">v{r.version}</span>
                      </MenuItem>
                    ))}
                  </Select>
                </FormControl>

                <Button
                  variant="contained"
                  disableElevation
                  fullWidth
                  sx={{ mt: 2 }}
                  startIcon={<DownloadIcon />}
                  disabled={selectedRelease == null}
                  onClick={() => {
                    if (selectedRelease == null) return
                    setShowInstaller(true)
                  }}
                >
                  Install
                </Button>
              </CardContent>
            </Card>
          </Grid>
        </Grid>
      </Container>

      <MaInstallDialog
        product={product}
        release={selectedRelease}
        open={showInstaller}
        onClose={() => setShowInstaller(false)}
      />
    </>
  )
}
